#include "usplash_backend.h"
#include "svgalib/gl/vgagl.h"
#include "svgalib/src/vga.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* 640x480 */
static int svgalib_mode = 10;

struct vesa_mode {
	int x;
	int y;
	int mode;
};

/* Table of 256-colour modes for now  - get from /usr/include/vga.h */

struct vesa_mode vesa_modes[] = {
	{ 640, 480, 10 },
	{ 800, 600, 11 },
	{ 1024, 768, 12 },
	{ 1152, 864, 39 },
	{ 1280, 1024, 13 },
	{ 1600, 1200, 45 },
	{ 1920, 1440, 70 },
	{ -1, -1, -1 },
};

struct usplash_pixmap
{
	int width, height;                  /* Width, height in pixels. */
	int ncols;                          /* Number of colors. */
	int transparent;                    /* Transparent color or -1 if none. */
	unsigned char (*palette)[3];        /* Palette. */
	unsigned char *data;                /* Run-length compressed data. */
};


int usplash_setfont (void* font) {
	// FIXME	
	gl_setfont(8, 8, gl_font8x8);
	gl_setwritemode(FONT_COMPRESSED + WRITEMODE_OVERWRITE);
	return 0;
}

int usplash_init () {
	int rc = 0;
	
	vga_setchipset (VESA);

	rc = vga_init ();
	
	if (rc) {
		return rc;
	}
	
	rc = vga_setmode (svgalib_mode);

	if (rc) {
		return rc;
	}

	return gl_setcontextvga(svgalib_mode);
}

void usplash_set_resolution (int x, int y) {
	int i=0;

	while (vesa_modes[i].y != -1) {
		if (vesa_modes[i].y > y)
			break;
		svgalib_mode = vesa_modes[i].mode;
		i++;
	}
}

void usplash_clear (int x1, int y1, int x2, int y2, int colour) {
	gl_fillbox (x1, y1, x2-x1, y2-y1, colour);
}

void usplash_move (int sx, int sy, int dx, int dy, int w, int h) {
	gl_copybox (sx, sy, w, h, dx, dy);
}

void usplash_text (int x, int y, const char *s, int len, int fg, int bg) {
	gl_setfontcolors (bg, fg);
	gl_writen (x, y, len, s);
}

void usplash_put (int x, int y, void *pointer) {
	struct usplash_pixmap *pixmap = pointer;
	unsigned char *decoded_pixmap;
	unsigned char *encoded_pixmap = pixmap->data;
	unsigned char *dst, *src;
	int h, w;
	
	
	decoded_pixmap = malloc (pixmap->height * pixmap->width * sizeof(char));
	dst = decoded_pixmap;
	src = encoded_pixmap;
	/* Oh god the misery */
	h = pixmap->height;
	while (h--) {
		int w = pixmap->width;
		int offset = 0;
		while (w)
		{
			int i;
			int color = *src & 0xf;
			int count = *src >> 4;
			src++;
			w -= count;

			memset (dst+offset, color, count);
			offset += count;
		}
		dst += pixmap->width;
	}


	gl_putbox (x, y, pixmap->width, pixmap->height, decoded_pixmap);
}

void usplash_done () {
	return;
}

void usplash_set_palette (int ncols, unsigned char palette[][3]) {
	gl_setpalettecolors (0, ncols, palette);
}

void usplash_getdimensions (int *x, int *y) {
	*x = WIDTH;
	*y = HEIGHT;
}

/* We don't need these. But svgalib wants them for linking purposes. MATTHEW
   SMASH */

double pow (double x, double y) {
	return 0;
}

double log (double x) {
	return 0;
}
